#include "config.h"

#include "gettext.h"
#include "music.h"
#include "main.h"
#include "playlist.h"

#include <xine.h>
#include <glib.h>
#include <string.h>
#include <stdarg.h>
#include <stdlib.h>

static xine_t             *xine;
static xine_stream_t      *stream;
static xine_audio_port_t  *ao;
static xine_video_port_t  *vo;
static xine_event_queue_t *events;

static int                 stream_pos;

static GList *notify_channels = NULL;
gboolean music_playing = FALSE;
static gboolean paused = FALSE;

void
music_events (void *data, const xine_event_t *e)
{ 
    thread_lock ();

    if (main_status == QUARK_RUNNING) {

        g_message ("XINE EVENT %d", e->type);

        switch (e->type) {
        case XINE_EVENT_UI_PLAYBACK_FINISHED:
            playlist_advance (1, main_loop_at_end);
            break;
        case XINE_EVENT_MRL_REFERENCE:
            g_message ("MRL REFERENCE");
            break;
        case XINE_EVENT_UI_MESSAGE:
            g_message (_("Message from Xine: %s"),
                       (char*) e->data);
            break;
        }
    }

    thread_unlock ();
}

void
music_init ()
{
    if (!xine_check_version (XINE_MAJOR_VERSION, XINE_MINOR_VERSION,
                             XINE_SUB_VERSION)) {
        g_critical (_("Incompatible version of Xine-lib found."));
        exit (EXIT_FAILURE);
    }

    xine = xine_new ();

    xine_init (xine);

    if (!(ao = xine_open_audio_driver (xine, NULL, NULL))) {
        g_critical (_("Unable to open audio driver from Xine."));
        exit (EXIT_FAILURE);
    }

    if (!(vo = xine_open_video_driver (xine, NULL, XINE_VISUAL_TYPE_NONE,
                                       NULL))) {
        g_critical (_("Unable to open video driver from Xine."));
        exit (EXIT_FAILURE);
    }

    if (!(stream = xine_stream_new (xine, ao, vo))) {
        g_critical (_("Unable to open a Xine stream."));
        exit (EXIT_FAILURE);
    }

    events = xine_event_new_queue (stream);
    xine_event_create_listener_thread (events, music_events, NULL);
}


void
music_destroy ()
{
    xine_event_dispose_queue (events);
    xine_dispose (stream);
    xine_close_video_driver (xine, vo);
    xine_close_audio_driver (xine, ao);
    xine_exit (xine);
}

void
music_play ()
{
    PlaylistItem *item;
    gint state, pos;
    gchar *path;

    state = xine_get_status (stream);

    if (!music_playing && playlist_current) {
        if (!paused) {
            item = LISTITEM (playlist_current);

            if (!(path = g_filename_from_utf8 (PATH (item), -1,
                                               NULL, NULL, NULL))) {
                g_critical (_("Skipping '%s'. Could not convert from UTF-8. "
                              "Bug?"), PATH (item));
                playlist_fail ();
                return;
            } else if (!xine_open (stream, path)) {
                g_message ("OPEN FAILED");
                playlist_fail ();
                return;
            }

            music_notify ("playing: %s (%s)", MAIN_PATH (item), PATH (item));
        }
        
        pos = stream_pos;
        stream_pos = 0;
        if (xine_play (stream, pos, 0)) {
            music_playing = TRUE;
            paused = FALSE;
        } else {
            playlist_fail ();
            g_message ("PLAY FAILED");
        }
    }
}

void
music_pause ()
{
    int state;

    state = xine_get_status (stream);
    if (state == XINE_STATUS_PLAY) {
        int pos, time, length;
        /* length = 0 for streams */
        if (!(xine_get_pos_length (stream, &pos, &time, &length) && length))
            pos = 0;
        stream_pos = pos;
        xine_stop (stream);
        paused = TRUE;
    }
    music_playing = FALSE;
}

void
music_stop ()
{
    int state;

    state = xine_get_status (stream);
    if (state != XINE_STATUS_IDLE)
        xine_close (stream);
    music_playing = FALSE;
    paused = FALSE;
}

void
music_add_notify (GIOChannel *output)
{
    notify_channels = g_list_prepend (notify_channels, output);
}

gboolean
music_list_notify (GIOChannel *channel, const char *message)
{
    gsize written;
    GIOStatus status;
    gboolean retval = FALSE;

    while ( (status = g_io_channel_write_chars (channel, message,
                                                strlen (message),
                                                &written, NULL))
            == G_IO_STATUS_AGAIN) {
        message += written;
    }
    
    if (status == G_IO_STATUS_ERROR) {
        retval = TRUE;
    }
    while ( (status = g_io_channel_flush(channel, NULL)) == G_IO_STATUS_AGAIN);

    if (status == G_IO_STATUS_ERROR)
        retval = TRUE;

    return retval;
}
    
void
music_notify (const char *format, ...)
{
    char *message, *newformat;
    GList *it;
    va_list args;

    va_start (args, format);
    
    message = g_strdup_vprintf(format, args);
    g_message ("%s", message);
    g_free (message);
            
    newformat = g_strdup_printf ("%s\n", format);
    message = g_strdup_vprintf (newformat, args);
    it = notify_channels;

    while (it != NULL) {
        if (music_list_notify (it->data, message)) {
            GList *tmp = it->next;
            g_list_remove_link (notify_channels, it);
            g_io_channel_shutdown (it->data, FALSE, NULL);
            g_io_channel_unref (it->data);
            if (it == notify_channels) {
                notify_channels = NULL;
            }
            it = tmp;
        } else {
            it = it->next;
        }
    }
    g_free (message);
    g_free (newformat);
    
    va_end (args);
}
